@@ -16,7 +16,7 @@ impl Default for ReadPositionBias {
 }
 
 impl Bias for ReadPositionBias {
-    fn prob(&self, observation: &Observation<ReadPosition>) -> LogProb {
+    fn prob_alt(&self, observation: &Observation<ReadPosition>) -> LogProb {
         match (self, observation.read_position) {
             (ReadPositionBias::None, _) => observation.prob_hit_base, // normal
             (ReadPositionBias::Some, ReadPosition::Major) => LogProb::ln_one(), // bias
